
function OUCN:OnInitialize()
  SLASH_OUCN1 = "/oucn";
  SlashCmdList["OUCN"] = function(msg)
    OUCN:ExecCmd(msg);
  end

  self:RegisterEvent("PLAYER_ENTERING_WORLD");
  self:RegisterEvent("VARIABLES_LOADED");

  self:SecureHook("UseAction");
  self:SecureHook("UseInventoryItem");
  self:SecureHook("CastPetAction");
  self:SecureHook("CastSpell");
  self:SecureHook("CastSpellByName");
end

function OUCN:OnEnable()
end

function OUCN:OnDisable()
end


function OUCN:VARIABLES_LOADED()
  if (VipersAddonsLoaded) then
    table.insert(VipersAddonsLoaded, {
      NAME = OUCN.vars.name,
      VERSION = OUCN.vars.version,
      OPTIONSFRAME = nil,
    });
  end

  if ((not VipersAddonsSettings) or ((VipersAddonsSettings) and (not VipersAddonsSettings["SURPRESSLOADMSG"])) and (DEFAULT_CHAT_FRAME)) then
    DEFAULT_CHAT_FRAME:AddMessage("|cffffffff- |cff00f100Viper's "..OUCN.vars.name.." is loaded (version "..OUCN.vars.version..").");
  end

  -- Setup global settings
  if (not oucnSettings) then
    oucnSettings = {};
  end
  OUCN:SetKeys(OUCN.vars.defaults, oucnSettings);

  -- Setup per-character settings
  if (not oucnSettings[OUCN.vars.realmName]) then
    oucnSettings[OUCN.vars.realmName] = {};
  end
  if (not oucnSettings[OUCN.vars.realmName][OUCN.vars.playerName]) then
    oucnSettings[OUCN.vars.realmName][OUCN.vars.playerName] = {};
  end
  for i, v in pairs(OUCN.vars.perCharacterSettings) do
    if (not oucnSettings[OUCN.vars.realmName][OUCN.vars.playerName][i]) then
      oucnSettings[OUCN.vars.realmName][OUCN.vars.playerName][i] = OUCN.vars.defaults[i];
    end
  end
  OUCN.charSettings = oucnSettings[OUCN.vars.realmName][OUCN.vars.playerName];

  -- Set cast ignore time depending on showGCD state.
  if (oucnSettings.showGCD) then
    OUCN.vars.castIgnoreTime = oucnSettings.abilityIgnoreTimes.GCD;
  else
    OUCN.vars.castIgnoreTime = oucnSettings.abilityIgnoreTimes.default;
  end

  OUCN:CreateFrames();
end


function OUCN:PLAYER_ENTERING_WORLD()
  OUCN:IndexActions();
  OUCN:SetGCDtime();
  OUCN.vars.enabled = true;
end


function OUCN:SetOnUpdate(frame, on)
  local frameRef = OUCN.vars.frames[frame].main;
  if (on) then
    if (frame == OUCN.vars.strings.countdown) then
      frameRef:SetScript("OnUpdate", function() OUCN:OnUpdate(arg1) end);
    elseif (frame == OUCN.vars.strings.list) then
      frameRef:SetScript("OnUpdate", function() OUCN:List_OnUpdate(arg1) end);
    elseif (frame == OUCN.vars.strings.updater) then
      frameRef:SetScript("OnUpdate", function() OUCN:Updater_OnUpdate(arg1) end);
    end
  else
    frameRef:SetScript("OnUpdate", nil);
  end
end


function OUCN:SetKeys(tbl, settingsTbl)
  local i, v;
  for i, v in pairs(tbl) do
    if (not OUCN.vars.perCharacterSettings[i]) then
      if (type(v) == "table") then
        if (settingsTbl[i] == nil) then
          settingsTbl[i] = v;
        else
          OUCN:SetKeys(v, settingsTbl[i]);
        end
      elseif (settingsTbl[i] == nil) then
        settingsTbl[i] = v;
      end
    end
  end
end


function OUCN:ExecCmd(s)
  local orgStr = tostring(s);
  local cmdStr = strupper(tostring(s));
  local cmds = {};
  local orgCmds = {};
  for v in string.gmatch(cmdStr, "[^ ]+") do
    table.insert(cmds, v);
  end
  for v in string.gmatch(orgStr, "[^ ]+") do
    table.insert(orgCmds, v);
  end

  local cmd = cmds[1];
  if (cmd == "M" or cmd == "MOVE") then
    if (OUCN.vars.allowMove) then
      OUCN:List_SetFrameVars();
      OUCN.vars.frames.countdown.main:EnableMouse(false);
      OUCN.vars.frames.countdown.main:Hide();
      OUCN.vars.frames.ability.main:EnableMouse(false);
      OUCN.vars.frames.ability.main:Hide();
      OUCN.vars.frames.list.main:EnableMouse(false);
      OUCN.vars.frames.list.main:Hide();
      OUCN.vars.allowMove = false;
      OUCN:ChatMsg("Windows are now locked.");
    else
      for n=1, OUCN.vars.cooldownListMax do
        OUCN.vars.frames.list["text"..n]:SetText("Cooldown Watch List item #"..n);
        OUCN.vars.frames.list["text"..n]:SetTextColor(1, 1, 1, 1);
      end
      OUCN.vars.frames.ability.text:SetText(OUCN.vars.name);
      OUCN.vars.frames.countdown.text:SetText(OUCN.vars.version);
      OUCN.vars.frames.countdown.main:SetAlpha(1);
      OUCN.vars.frames.ability.main:SetAlpha(1);
      OUCN.vars.allowMove = true;
      OUCN.vars.frames.countdown.main:EnableMouse(true);
      OUCN.vars.frames.ability.main:EnableMouse(true);
      OUCN.vars.frames.list.main:EnableMouse(true);
      OUCN.vars.frames.countdown.main:Show();
      OUCN.vars.frames.ability.main:Show();
      OUCN.vars.frames.list.main:Show()
      OUCN:ChatMsg("You can now move the windows.");
    end
  elseif (cmd == "D" or cmd == "DISABLE") then
    OUCN.vars.enabled = false;
    OUCN:ChatMsg("Add-on is now |cffff1010disabled|r.");
  elseif (cmd == "E" or cmd == "ENABLE") then
    OUCN.vars.enabled = true;
    OUCN:ChatMsg("Add-on is now |cff10ff10enabled|r.");
  elseif (cmd == "S" or cmd == "SHOWABILITY") then
    if (oucnSettings.showAbility) then
      OUCN.vars.frames.ability.main:Hide();
      oucnSettings.showAbility = false;
      OUCN:ChatMsg("No longer shows the name of the ability you tried to use.");
    else
      OUCN.vars.frames.ability.main:Show();
      oucnSettings.showAbility = true;
      OUCN:ChatMsg("Now shows the name of the ability you tried to use.");
    end
  elseif (cmd == "I" or cmd == "INDEX") then
    OUCN.vars.indexed = false;
    OUCN.vars.actions = {};
    OUCN:IndexActions();
    OUCN:ChatMsg("All spells/abilities are now indexed.");
  elseif (cmd == "R" or cmd == "FRACTIONS") then
    if (oucnSettings.useFractions) then
      oucnSettings.useFractions = false;
      OUCN:ChatMsg("No longer shows fraction of a second when the cooldown is less than 10 seconds.");
    else
      oucnSettings.useFractions = true;
      OUCN:ChatMsg("Now shows fraction of a second when the cooldown is less than 10 seconds.");
    end
  elseif (cmd == "T" or cmd == "TIMETYPE") then
    if (oucnSettings.useHMS) then
      oucnSettings.useHMS = false;
      OUCN:ChatMsg("No longer shows h/m/s after the cooldown.");
    else
      oucnSettings.useHMS = true;
      OUCN:ChatMsg("Now shows h/m/s after the cooldown.");
    end
  elseif (cmd == "C" or cmd == "COLOR") then
    if (oucnSettings.useColors) then
      oucnSettings.useColors = false;
      OUCN:ChatMsg("No longer uses coloring for the different time states.");
    else
      oucnSettings.useColors = true;
      OUCN:ChatMsg("Now uses coloring for the different time states.");
    end
  elseif (cmd == "A" or cmd == "COLORFADE") then
    if (oucnSettings.useColorFade) then
      oucnSettings.useColorsFade = false;
      OUCN:ChatMsg("No longer uses color fade for cooldowns below 10 seconds.");
    else
      oucnSettings.useColorFade = true;
      OUCN:ChatMsg("Now uses color fade for cooldowns below 10 seconds.");
    end
  elseif (cmd == "V" or cmd == "VISIBLE") then
    if (cmds[2]) then
      if (tonumber(cmds[2]) and tonumber(cmds[2]) > oucnSettings.main.fadeOutTime and tonumber(cmds[2]) < 10) then
        oucnSettings.main.timeToShow = tonumber(cmds[2]);
        OUCN:ChatMsg("New visible time set to: "..cmds[2]..".");
      else
        OUCN:ChatMsg("The value given is not valid. It must be higher than "..oucnSettings.main.fadeOutTime.." and lower than 10.");
      end
    end
  elseif (cmd == "O" or cmd == "COLLECT") then
    if (OUCN.vars.collectListIDs) then
      OUCN.vars.frames.ability.main:Hide();
      OUCN.vars.collectListIDs = false;
      OUCN:ChatMsg("Collect mode has been |cffff1010disabled|r.");
    else
      OUCN.vars.collectListIDs = true;
      OUCN.vars.frames.ability.text:SetText("Collect mode");
      OUCN.vars.frames.ability.text:SetTextColor(1, 1, 1, 1);
      OUCN.vars.frames.ability.main:SetAlpha(1);
      OUCN.vars.frames.ability.main:Show();
      OUCN:ChatMsg("Collect mode has been |cff10ff10enabled|r.");
    end
  elseif (cmd == "G" or cmd == "GCD") then
    if (oucnSettings.showGCD) then
      oucnSettings.showGCD = false;
      OUCN.vars.castIgnoreTime = oucnSettings.abilityIgnoreTimes.default;
      OUCN:ChatMsg("No longer shows when Global Cooldown is in effect.");
    else
      oucnSettings.showGCD = true;
      OUCN.vars.castIgnoreTime = oucnSettings.abilityIgnoreTimes.GCD;
      OUCN:ChatMsg("Now shows when Global Cooldown is in effect.");
    end
  elseif (cmd == "P" or cmd == "PREFIX") then
    if (oucnSettings.usePrefix) then
      oucnSettings.usePrefix = false;
      OUCN:ChatMsg("No longer shows prefix in front of ability name.");
    else
      oucnSettings.usePrefix = true;
      OUCN:ChatMsg("Now shows prefix in front of ability name.");
    end
  elseif (cmd == "F" or cmd == "FONT") then
    local subCmd = cmds[2];
    if (subCmd == "F" or subCmd == "FACE") then
      if (cmds[3] and tonumber(cmds[3]) and cmds[4]) then
        local n = tonumber(cmds[3]);
        local frame = strlower(cmds[4]);
        local fonts = OUCN.SM:List(OUCN.SM.MediaType.FONT);
        local handle;
        for i, v in pairs(fonts) do
          if (i == n) then
            handle = v;
          end
        end
        if (not handle) then
          return;
        end
        local font = OUCN.SM:Fetch(OUCN.SM.MediaType.FONT, handle);
        if (font) then
          if (frame == "list") then
            OUCN:SetFontFace(frame, font, true);
          else
            OUCN:SetFontFace(frame, font);
          end
          oucnSettings.font[frame] = handle;
        end
        OUCN:ChatMsg("Changed the font for frame \""..frame.."\" to: "..handle..".");
      else
        local fonts = OUCN.SM:List(OUCN.SM.MediaType.FONT);
        OUCN:ChatMsg("Available fonts through SharedMedia:", true);
        OUCN:ChatMsg("Type the number in front of the font name to change to that font (ie: \"/oucn f f 7 countdown\")", true);
        for i, v in pairs(fonts) do
          OUCN:ChatMsg("- "..i..": "..v, true);
        end
      end
    elseif (subCmd == "S" or subCmd == "SIZE") then
      if (cmds[3] and tonumber(cmds[3]) and cmds[4]) then
        local frame = strlower(cmds[4]);
        local size = tonumber(cmds[3]);
        oucnSettings.font.size[frame] = size;
        local list = false;
        if (frame == OUCN.vars.strings.list) then
          list = true;
        end
        OUCN:SetFontSize(frame, size, list);
        OUCN:ChatMsg("Changed the font size for frame \""..frame.."\" to: "..size..".");
      end
    else
      OUCN:ChatMsg("Commands available for \"FONT\":", true);
      OUCN:ChatMsg("You need to include the frame you wish to modify as the last parameter.", true);
      OUCN:ChatMsg("Available frames are: \"countdown\", \"ability\" and \"list\".", true);
      OUCN:ChatMsg("- |cffffe210F|cff10ff10ace|r: Sets the font face.", true);
      OUCN:ChatMsg("- |cffffe210S|cff10ff10ize|r: Sets the size of the font.", true);
      OUCN:ChatMsg("Example: Typing \"/oucn f s 10 countdown\" will set the font size of the countdown to 10.", true);
    end
  elseif (cmd == "L" or cmd == "MODEL") then
    if (cmds[2]) then
      if (cmds[2] == "E" or cmds[2] == "ENABLE") then
        oucnSettings.readyFlash.countdown.show = true;
        OUCN:ChatMsg("Ready effect is now |cff10ff10enabled|r.");
      elseif (cmds[2] == "D" or cmds[2] == "DISABLE") then
        oucnSettings.readyFlash.countdown.show = false;
        OUCN:ChatMsg("Ready effect is now |cffff1010disabled|r.");
      elseif (cmds[2] == "S" or cmds[2] == "SCALE") then
        if (cmds[3] and tonumber(cmds[3])) then
          oucnSettings.readyFlash.countdown.scale = tonumber(cmds[3]);
          OUCN:ChatMsg("Ready effect scale changed to \""..cmds[3].."\".");
        end
      else
        local str = "";
        for n=2, #orgCmds do
          str = str..orgCmds[n].." ";
        end
        str = string.sub(str, 1, (string.len(str)-1));
        OUCN:SetModel(str);
        OUCN:DoReadyFlash(OUCN.vars.strings.countdown, 0, nil);
      end
    else
      OUCN:ChatMsg("Commands available for \"MODEL\":", true);
      OUCN:ChatMsg("- |cffffe210E|cff10ff10nable|r: Enables showing the ready effect.", true);
      OUCN:ChatMsg("- |cffffe210D|cff10ff10isable|r: Disables showing the ready effect.", true);
      OUCN:ChatMsg("- |cffffe210S|cff10ff10cale|r: Sets the scale of the ready effect. Default value is "..OUCN.vars.defaults.readyFlash.countdown.scale..".", true);
      OUCN:ChatMsg("Available models:", true);
      for i, v in pairs(OUCN.vars.readyFlash.models) do
        OUCN:ChatMsg("- \"|cff10ff10"..i.."|r\".", true);
      end
      OUCN:ChatMsg("Example: Typing \"/oucn l Battle Shout\" would change the model to use the one for Battle Shout.", true);
      OUCN:ChatMsg("Note that the model names are case-sensitive and has to be written exactly as they're listed.", true);
    end
  else
    OUCN:ChatMsg(OUCN.vars.name.." (version "..OUCN.vars.version..") command list.", true);
    OUCN:ChatMsg("Type in the highlighted letter or the full word to activate the command.", true);
    OUCN:ChatMsg("- |cffffe210M|cff10ff10ove|r: Allows moving the window. Type again to lock it in place.", true);
    OUCN:ChatMsg("- |cffffe210D|cff10ff10isable|r: Disables the add-on.", true);
    OUCN:ChatMsg("- |cffffe210E|cff10ff10nable|r: Enables the add-on.", true);
    OUCN:ChatMsg("- |cffffe210S|cff10ff10howability|r: Toggles showing the name of the ability you're trying to use.", true);
    OUCN:ChatMsg("- |cffffe210I|cff10ff10ndex|r: Re-indexes the abilites in your action bars for displaying when on cooldown. Use this after you move any of your spells/abilities.", true);
    OUCN:ChatMsg("- |cff10ff10F|cffffe210r|cff10ff10actions|r: Toggles showing fraction of a second when the cooldown is less than 10 seconds.", true);
    OUCN:ChatMsg("- |cffffe210T|cff10ff10imetype|r: Toggles showing the time type; h/m/s (hour/minute/second), after the cooldown count.", true);
    OUCN:ChatMsg("- |cffffe210C|cff10ff10olor|r: Toggles coloring of the cooldown based on what state it's in (hour, minute, second, fraction).", true);
    OUCN:ChatMsg("- |cff10ff10Colorf|cffffe210a|cff10ff10de|r: Toggles a color fade going from red (at 10 seconds) to green (at 0) while the cooldown is below 10 seconds.", true);
    OUCN:ChatMsg("- |cffffe210V|cff10ff10isible X|r: Sets the time in seconds (can also include fraction of a second) that the cooldown should be visible. The default value is "..OUCN.vars.defaults.main.timeToShow.." seconds.", true);
    OUCN:ChatMsg("- |cff10ff10C|cffffe210o|cff10ff10llect|r: Toggles the collect ID mode. When enabled you can click spells/abilities in your action bars to add them to the cooldown watch list. Only a maximum of "..OUCN.vars.cooldownListMax.." can be stored.", true);
    OUCN:ChatMsg("- |cffffe210G|cff10ff10CD|r: Toggles showing the Global Cooldown when in effect.", true);
    OUCN:ChatMsg("- |cffffe210P|cff10ff10refix|r: Toggles showing pet/item prefix in front of ability name.", true);
    OUCN:ChatMsg("- |cffffe210F|cff10ff10ont|r: Access to font options.", true);
    OUCN:ChatMsg("- |cff10ff10Mode|cffffe210l|cff10ff10|r: Change the model used for the ready effect. The ready effect will be shown when a visible countdown reaches zero.", true);
  end
end


function OUCN:SetValue(key, value)
  if (oucnSettings[key] ~= nil) then
    oucnSettings[key] = value;
  else
    OUCN:ChatMsg("The key (\""..tostring(key).."\") doesn't exist in \"oucnSettings\".");
  end
end


function OUCN:ChatMsg(s, compact)
  if (DEFAULT_CHAT_FRAME) then
    if (compact) then
      DEFAULT_CHAT_FRAME:AddMessage(s, 1.0, 0.8, 0.2);
    else
      DEFAULT_CHAT_FRAME:AddMessage("|cffdfa322["..OUCN.vars.name.."]|r "..s, 1.0, 0.8, 0.2);
    end
  end
end


function OUCN:OnDragStart()
  this:StartMoving();
  OUCN.vars.isMoving = true;
end

function OUCN:OnDragStop()
  if (OUCN.vars.isMoving) then
    this:StopMovingOrSizing();
    local point, relativeTo, relativePoint, offsetX, offsetY = this:GetPoint();
    local relativeFrame = nil;
    if (relativeTo) then
      relativeFrame = relativeTo:GetName();
    end
    OUCN.charSettings.framePositions[this:GetName()] = {point, relativeFrame, relativePoint, offsetX, offsetY};
    OUCN.vars.isMoving = false;
  end
end


function OUCN:GetFontFace(frame)
  return OUCN.SM:Fetch(OUCN.SM.MediaType.FONT, oucnSettings.font[frame]);
end

function OUCN:GetFontSize(frame)
  return oucnSettings.font.size[frame];
end

function OUCN:GetFontOutline(frame)
  return oucnSettings.font.outline[frame];
end


function OUCN:SetFontFace(frame, font, list)
  if (list) then
    for n=1, OUCN.vars.cooldownListMax do
      OUCN.vars.frames.list["text"..n]:SetFont(font, OUCN:GetFontSize(frame), OUCN:GetFontOutline(frame));
    end
  else
    OUCN.vars.frames[frame].text:SetFont(font, OUCN:GetFontSize(frame), OUCN:GetFontOutline(frame));
  end
end

function OUCN:SetFontSize(frame, size, list)
  if (list) then
    for n=1, OUCN.vars.cooldownListMax do
      OUCN.vars.frames.list["text"..n]:SetFont(OUCN:GetFontFace(frame), size, OUCN:GetFontOutline(frame));
    end
    OUCN.vars.frames.list.main:SetHeight((size*OUCN.vars.cooldownListMax)+(OUCN.vars.list.spacing*OUCN.vars.cooldownListMax));
  else
    OUCN.vars.frames[frame].text:SetFont(OUCN:GetFontFace(frame), size, OUCN:GetFontOutline(frame));
    OUCN.vars.frames[frame].main:SetHeight(size);
  end
end


function OUCN:GetStatusbarTexture()
  return OUCN.SM:Fetch(OUCN.SM.MediaType.STATUSBAR, oucnSettings.watcher.texture);
end

function OUCN:GetStatusbarColor()
  local color= oucnSettings.watcher.color;
  local a = oucnSettings.watcher.alpha;
  return color.r, color.g, color.b, a;
end


function OUCN:CreateFrames()
  CreateFrame("MODEL", format(OUCN.vars.strings.templates.main, "ReadyFlash"), UIParent);
  CreateFrame("GAMETOOLTIP", "OnUseCooldownNotifierTooltip", UIParent, "GameTooltipTemplate");
  CreateFrame("FRAME", format(OUCN.vars.strings.templates.main, "Updater"), UIParent);
  OUCN:CreateFrame("Countdown", 150, 1, 0, 0);
  OUCN:CreateFrame("Ability", 500, 1, 0, 30);
  OUCN:CreateFrame("List", 350, OUCN.vars.cooldownListMax, 0, -50);

  local f = OUCN:CreateFrame("Watcher", 500, 0, 0, 60);

  local fText = f:CreateFontString(format(OUCN.vars.strings.templates.child, "Watcher", "Countdown"), "OVERLAY");
  fText:SetPoint("RIGHT");
  fText:SetTextColor(1, 1, 1, 1);
  fText = f:CreateFontString(format(OUCN.vars.strings.templates.child, "Watcher", "Ability"), "OVERLAY");
  fText:SetPoint("LEFT");
  fText:SetTextColor(1, 1, 1, 1);

  f = CreateFrame("STATUSBAR", format(OUCN.vars.strings.templates.child, "Watcher", "Statusbar"), getglobal(format(OUCN.vars.strings.templates.main, "Watcher")));
  f:SetStatusBarTexture(OUCN:GetStatusbarTexture());
  f:SetStatusBarColor(OUCN:GetStatusbarColor());
  f:Hide();

  for i, v in pairs(OUCN.vars.frames) do
    if (type(v) == "string") then
      OUCN.vars.frames[i] = getglobal(v);
    elseif (type(v) == "table") then
      for n, k in pairs(v) do
        v[n] = getglobal(k);
      end
    end
  end

  OUCN:SetFontSize(OUCN.vars.strings.countdown, oucnSettings.font.size.countdown, false);
  OUCN:SetFontSize(OUCN.vars.strings.ability, oucnSettings.font.size.ability, false);
  OUCN:SetFontSize(OUCN.vars.strings.list, oucnSettings.font.size.list, true);

  OUCN:List_SetSpacing();
  OUCN:List_SetFrameVars();

  OUCN.vars.frames.countdown.main.delayer = 0;
  OUCN.vars.frames.countdown.main.alphaTime = 0;
  OUCN.vars.frames.list.main.delayer = 0;

  OUCN.vars.frames.countdown.main:SetScript("OnShow", function() OUCN:SetOnUpdate(OUCN.vars.strings.countdown, true) end);
  OUCN.vars.frames.countdown.main:SetScript("OnHide", function() OUCN:SetOnUpdate(OUCN.vars.strings.countdown, false) end);
  OUCN.vars.frames.list.main:SetScript("OnShow", function() OUCN:SetOnUpdate(OUCN.vars.strings.list, true) end);
  OUCN.vars.frames.list.main:SetScript("OnHide", function() OUCN:SetOnUpdate(OUCN.vars.strings.list, false) end);
end


function OUCN:CreateFrame(name, width, textCount, offsetX, offsetY)
  local fText, frameNo;
  local f = CreateFrame("FRAME", format(OUCN.vars.strings.templates.main, name), UIParent);
  f:SetFrameStrata("BACKGROUND");
  f:SetWidth(width);
  f:SetToplevel(true);
  f:SetMovable(true);
  f:EnableMouse(false);
  f:SetPoint("CENTER", UIParent, "CENTER", offsetX, offsetY);
  for n=1, textCount do
    frameNo = "";
    if (textCount > 1) then
      frameNo = tostring(n);
    end
    fText = f:CreateFontString(format(OUCN.vars.strings.templates.text, name, frameNo), "OVERLAY");
    fText:SetPoint("CENTER");
    fText:SetTextColor(1, 1, 1, 1);
    fText:SetJustifyH("CENTER");
  end
  f:RegisterForDrag("LeftButton");
  f:SetScript("OnDragStart", function() OUCN:OnDragStart() end);
  f:SetScript("OnDragStop", function() OUCN:OnDragStop() end);
  f:SetScript("OnMouseUp", function() OUCN:OnDragStop() end);
  f:Hide();
  return f;
end


function OUCN:SetGCDtime()
  if (OUCN.vars.GCDtimes[strlower(UnitClass("player"))]) then
    OUCN.vars.GCDtime = OUCN.vars.GCDtimes[strlower(UnitClass("player"))];
  else
    OUCN.vars.GCDtime = OUCN.vars.GCDtimes.default;
  end
end


function OUCN:SetModel(s)
  if (OUCN.vars.readyFlash.models[s]) then
    oucnSettings.readyFlash.countdown.model = s;
    OUCN:ChatMsg("Changed model to \"|cff10ff10"..s.."|r\".");
  else
    OUCN:ChatMsg("Can't find the model \"|cffff1010"..tostring(s).."|r\".");
  end
end
